﻿Imports System.Drawing.Drawing2D
Public Class Form1

    ' 目盛りを表示する
    Sub DrawScale(ByVal DrawBitmap As Bitmap, ByVal DayInMonth As Integer)

        Dim g As Graphics = Graphics.FromImage(DrawBitmap)

        g.SmoothingMode = SmoothingMode.AntiAlias

        Const ScaleSize As Integer = 4

        Dim y As Integer = DrawBitmap.Size.Height \ 2
        g.DrawLine(New Pen(Color.Black, 1), 0, y, DrawBitmap.Size.Width, y)

        Dim i As Integer

        For i = 1 To DayInMonth - 1
            Dim x As Integer = DrawBitmap.Size.Width * i / DayInMonth
            g.DrawLine(New Pen(Color.Black, 1), x, y - ScaleSize, x, y + ScaleSize)
        Next

        Dim MyFont As Font = New Font("Arial", 8)
        Dim fs As SizeF = g.MeasureString("0", MyFont)
        g.TextRenderingHint = Drawing.Text.TextRenderingHint.AntiAliasGridFit

        Const FontMargin As Integer = 4

        For i = 1 To DayInMonth
            Dim x As Integer = DrawBitmap.Size.Width / DayInMonth * (i - 0.5) - fs.Width / 2
            fs = g.MeasureString(i, MyFont)
            g.DrawString(i, MyFont, New SolidBrush(Color.Black), x, y + FontMargin - (i Mod 2) * (fs.Height + FontMargin * 2))
        Next

    End Sub

    ' カーブを表示する
    Sub DisplayCurve(ByVal DrawBitmap As Bitmap, ByVal Span As Integer, ByVal Value As Integer, ByVal DayInMonth As Integer, ByVal linecolor As Color)

        Dim Bsize As Size = DrawBitmap.Size

        Dim x, y As Integer
        Dim m As Integer = (Value Mod Span) * (Bsize.Width / DayInMonth)
        Dim Points() As Point = New Point(Bsize.Width) {}

        Dim Factor As Double = (Math.PI * 2) / ((Bsize.Width / DayInMonth) * Span)

        For x = 0 To Bsize.Width
            y = -Math.Sin((x + m) * Factor) * Bsize.Height * 0.5 * 0.95 + Bsize.Height * 0.5
            Points(x) = New Point(x, y)
        Next

        Dim g As Graphics = Graphics.FromImage(DrawBitmap)
        g.SmoothingMode = SmoothingMode.AntiAlias

        g.DrawLines(New Pen(linecolor, 2), Points)

    End Sub

    ' バイオリズムを表示する
    Sub DrawBio()

        Dim BioBitmap As Bitmap = New Bitmap(BioPictureBox.ClientSize.Width, BioPictureBox.ClientSize.Height)
        Dim DrawDays As Integer = Date.DaysInMonth(DrawDateTimePicker.Value.Year, DrawDateTimePicker.Value.Month)

        Dim BirthDay As Date
        Try
            BirthDay = GetBirthDate()
        Catch ex As ArgumentOutOfRangeException
            MessageBox.Show("正しい誕生日を入力してください", Me.Text, MessageBoxButtons.OK, MessageBoxIcon.Error)
            Exit Sub
        End Try

        Dim StartDate As Date = New Date(DrawDateTimePicker.Value.Year, DrawDateTimePicker.Value.Month, 1)

        Dim SpanDays As Integer = StartDate.Subtract(BirthDay).TotalDays

        DisplayCurve(BioBitmap, 23, SpanDays, DrawDays, Color.Blue)
        DisplayCurve(BioBitmap, 28, SpanDays, DrawDays, Color.Red)
        DisplayCurve(BioBitmap, 33, SpanDays, DrawDays, Color.Green)
        DrawScale(BioBitmap, DrawDays)

        BioPictureBox.Image = BioBitmap

        ' 日付を表示する

        Dim S() As String = MakeTable(23, SpanDays, DrawDays)
        PTextBox.Text = "高調" & vbCrLf & S(0) & vbCrLf & "低調" & vbCrLf & S(1) & vbCrLf & "注意" & vbCrLf & S(2)
        S = MakeTable(28, SpanDays, DrawDays)
        STextBox.Text = "高調" & vbCrLf & S(0) & vbCrLf & "低調" & vbCrLf & S(1) & vbCrLf & "注意" & vbCrLf & S(2)
        S = MakeTable(33, SpanDays, DrawDays)
        ITextBox.Text = "高調" & vbCrLf & S(0) & vbCrLf & "低調" & vbCrLf & S(1) & vbCrLf & "注意" & vbCrLf & S(2)

    End Sub

    Private Sub DateTimePicker1_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles DrawDateTimePicker.ValueChanged

        DrawBio()

    End Sub

    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        SetDate()
        DrawBio()

    End Sub

    Function MakeTable(ByVal Span As Integer, ByVal Value As Integer, ByVal DayInMonth As Integer) As String()

        Dim Message() As String = New String(2) {}

        Dim d As Integer
        Dim p As Integer = Math.Floor(Span / 2)
        Dim c1 As Integer = 3
        Dim c2 As Integer = 3

        For d = 1 To DayInMonth + 1
            Dim n As Integer
            Select Case ((d + Value - 1) Mod Span)
                Case 0, p
                    ' 不安定期
                    n = 2
                Case Is > p
                    ' 低調期
                    n = 1
                Case Is > 0
                    ' 高調期
                    n = 0
            End Select

            If d > DayInMonth Then
                n = 3
            End If

            If c1 <> n Then
                If d <= DayInMonth Then
                    If Message(n) <> "" Then
                        Message(n) += "・"
                    End If
                    Message(n) += d & "日"
                End If
                If d <> 1 And c1 = c2 Then
                    Message(c1) += "～" & d - 1 & "日"
                End If
            End If

            c2 = c1
            c1 = n

        Next

        Return Message

    End Function

    Sub SetDate()

        ' コンボボックスに年月日を表示する

        Dim i As Integer

        Dim y As Integer = Date.Now.Year
        For i = y - 89 To y
            YearComboBox.Items.Add(i & "年")
        Next

        YearComboBox.SelectedIndex = 69

        For i = 1 To 12
            MonthComboBox.Items.Add(i & "月")
        Next

        MonthComboBox.SelectedIndex = Date.Now.Month - 1

        For i = 1 To 31
            DayComboBox.Items.Add(i & "日")
        Next

        DayComboBox.SelectedIndex = Date.Now.Day - 1

    End Sub

    Function GetBirthDate() As Date

        Dim y As Integer = TrimLastChar(YearComboBox.Text)
        Dim m As Integer = TrimLastChar(MonthComboBox.Text)
        Dim d As Integer = TrimLastChar(DayComboBox.Text)

        Return New Date(y, m, d)

    End Function

    Function TrimLastChar(ByVal s As String) As String

        Return s.Remove(s.Length - 1, 1)

    End Function
End Class
