﻿Public Class Form1

    Dim CellImages(,) As Image
    Dim CellPictureBox() As PictureBox

    ' 元のイメージを保存
    Dim MainImage As Image
    Dim SourceImage As Image

    ' ドラッグ&ドロップの位置補正
    Dim Delta As Point

    ' 現在の枠の状況
    Dim CurrentBorder As Boolean
    ' 縦横の分割数
    Dim XCount, YCount As Integer

    Private Sub PictureBoxes_MouseDown(ByVal sender As System.Object, ByVal e As System.Windows.Forms.MouseEventArgs)

        DrawCell(True)

        Delta = e.Location
        CType(sender, PictureBox).BringToFront()
        CType(sender, PictureBox).DoDragDrop(sender, DragDropEffects.Move)

    End Sub

    Sub MakePictureBox()

        ' 古いピクチャボックスを破棄する
        If CellPictureBox IsNot Nothing Then
            For i As Integer = 0 To CellPictureBox.GetUpperBound(0)
                CellPictureBox(i).Dispose()
            Next
        End If

        ' ピクチャボックスを生成
        Dim Bound As Integer = XCount * YCount - 1

        CellPictureBox = New PictureBox(Bound) {}

        For i As Integer = 0 To Bound
            Dim p As PictureBox = New PictureBox()
            AddHandler p.MouseDown, AddressOf PictureBoxes_MouseDown
            Me.Controls.Add(p)
            CellPictureBox(i) = p
            p.Name = "PictureBoxes" & i
            p.Tag = i
            p.BringToFront()
        Next

    End Sub

    Sub DrawFrame()

        Dim cx, cy As Integer
        Dim g As Graphics

        ' 枠のピクチャボックスのサイズとフォームの配置
        FramePictureBox.Width = (MainImage.Width \ XCount) * XCount + 1
        FramePictureBox.Height = (MainImage.Height \ YCount) * YCount + 1

        cx = Math.Max(8, (Me.ClientSize.Width - FramePictureBox.Width) \ 2)
        cy = Math.Max(8, (Me.ClientSize.Height - FramePictureBox.Height) \ 2)

        FramePictureBox.Location = New Point(cx, cy)

        ' 枠を書く
        FramePictureBox.Image = New Bitmap(FramePictureBox.ClientRectangle.Width, FramePictureBox.ClientRectangle.Height)
        g = Graphics.FromImage(FramePictureBox.Image)
        g.FillRectangle(New SolidBrush(Color.White), New Rectangle(0, 0, FramePictureBox.Image.Width, FramePictureBox.Image.Height))
        g.DrawRectangle(New Pen(Color.Black), New Rectangle(0, 0, FramePictureBox.Image.Width - 1, FramePictureBox.Image.Height - 1))
        g.Dispose()

    End Sub

    Private Sub Form1_DragDrop(ByVal sender As Object, ByVal e As System.Windows.Forms.DragEventArgs) Handles Me.DragDrop

        Dim a As Object = FramePictureBox
        Dim p As PictureBox = CType(e.Data.GetData(a.GetType), PictureBox)

        Dim l As Point = Me.PointToClient(New Point(e.X, e.Y)) - Delta
        p.Location = AdjustPiece(p.Tag(), l)

        ' 完成していれば枠を消去
        If IsComplete() Then
            DrawCell(False)
        End If

    End Sub

    Private Sub Form1_DragEnter(ByVal sender As Object, ByVal e As System.Windows.Forms.DragEventArgs) Handles Me.DragEnter

        e.Effect = DragDropEffects.Move

    End Sub

    Private Sub Form1_DragOver(ByVal sender As Object, ByVal e As System.Windows.Forms.DragEventArgs) Handles Me.DragOver

        Dim a As Object = FramePictureBox
        Dim p As PictureBox = CType(e.Data.GetData(a.GetType), PictureBox)

        e.Effect = DragDropEffects.Move
        p.Location = Me.PointToClient(New Point(e.X, e.Y)) - Delta

    End Sub

    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        XCount = 5
        YCount = 4
        MakeMainImage(My.Resources.Jigsaw)
        MakePictureBox()
        MakeCellImages()
        DrawFrame()
        Resolve()

    End Sub

    Sub MakeCellImages()

        ' 枠付きと枠なしのイメージを作成

        Dim Bound As Integer = XCount * YCount - 1
        Dim CellWidth As Integer = MainImage.Width \ XCount
        Dim CellHeight As Integer = MainImage.Height \ YCount

        CellImages = New Image(Bound, 1) {}

        For i As Integer = 0 To Bound
            CellImages(i, 0) = New Bitmap(CellWidth + 1, CellHeight + 1)
            CellImages(i, 1) = New Bitmap(CellWidth + 1, CellHeight + 1)
            Dim sr As Rectangle = New Rectangle(CellWidth * (i Mod XCount), CellHeight * (i \ XCount), CellWidth + 1, CellHeight + 1)

            ' 枠付きイメージ
            Dim g As Graphics = Graphics.FromImage(CellImages(i, 0))
            g.DrawImage(SourceImage, 0, 0, sr, GraphicsUnit.Pixel)
            g.DrawRectangle(New Pen(Color.Black), New Rectangle(0, 0, CellWidth, CellHeight))
            g.Dispose()

            ' 枠なしイメージ(大きな枠はある)
            g = Graphics.FromImage(CellImages(i, 1))
            g.DrawImage(SourceImage, 0, 0, sr, GraphicsUnit.Pixel)

            If i < XCount Then
                g.DrawLine(New Pen(Color.Black), 0, 0, CellWidth + 1, 0)
            End If
            If i Mod XCount = 0 Then
                g.DrawLine(New Pen(Color.Black), 0, 0, 0, CellHeight + 1)
            End If
            If (i Mod XCount) = (XCount - 1) Then
                g.DrawLine(New Pen(Color.Black), CellWidth, 0, CellWidth, CellHeight + 1)
            End If
            If i > Bound - XCount Then
                g.DrawLine(New Pen(Color.Black), 0, CellHeight, CellWidth + 1, CellHeight)
            End If
            g.Dispose()
        Next

    End Sub

    Sub DrawCell(ByVal Border As Boolean)

        If CurrentBorder = Border Then
            Exit Sub
        End If

        Dim Bound As Integer = XCount * YCount - 1
        Dim n As Integer = 1

        If Border Then
            n = 0
        End If

        For i As Integer = 0 To Bound
            CellPictureBox(i).ClientSize = CellImages(i, n).Size
            CellPictureBox(i).Image = CellImages(i, n)
        Next

        CurrentBorder = Border

    End Sub

    Sub Shuffle()

        Dim c As PictureBox
        Dim r As New Random()

        ' 枠を書く
        DrawFrame()

        ' セルをばらばらに
        For Each c In CellPictureBox
            c.Location = New Point(r.Next(Me.ClientSize.Width - c.Width - 4) + 2, r.Next(Me.ClientSize.Height - c.Height - 4) + 2)
        Next

        ' セルに枠を表示
        CurrentBorder = False
        DrawCell(True)

    End Sub

    Function AdjustPiece(ByVal i As Integer, ByVal p As Point) As Point

        Dim cp As Point = CorrectPoint(i)

        ' 正しいセルの位置との距離を計算
        Dim d As Integer = Math.Sqrt((p.X - cp.X) ^ 2 + (p.Y - cp.Y) ^ 2)

        ' 距離が16未満なら正しい位置に移動
        If d < 16 Then
            Return cp
        Else
            Return p
        End If

    End Function

    Function CorrectPoint(ByVal i As Integer) As Point

        ' セルの正しい位置を計算
        Dim CellWidth As Integer = MainImage.Width \ XCount
        Dim CellHeight As Integer = MainImage.Height \ YCount
        Dim cx, cy As Integer

        cx = (i Mod XCount) * CellWidth + FramePictureBox.Left
        cy = (i \ XCount) * CellHeight + FramePictureBox.Top

        Return New Point(cx, cy)

    End Function

    Public Function IsComplete() As Boolean

        Dim t As Boolean = True
        Dim i As Integer

        For Each c As PictureBox In CellPictureBox
            i = c.Tag
            If Not (CorrectPoint(i) = c.Location) Then
                t = False
                Exit For
            End If
        Next

        Return t

    End Function

    Private Sub ShuffleToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ShuffleToolStripMenuItem.Click

        Shuffle()

    End Sub

    Sub Resolve()

        For Each c As PictureBox In CellPictureBox
            c.Location = CorrectPoint(c.Tag)
        Next

        CurrentBorder = True
        DrawCell(False)

    End Sub

    Private Sub ResolveToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ResolveToolStripMenuItem.Click

        Resolve()

    End Sub

    Private Sub OpenToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles OpenToolStripMenuItem.Click

        Try
            If OpenFileDialog1.ShowDialog = Windows.Forms.DialogResult.OK Then
                MakeMainImage(Image.FromFile(OpenFileDialog1.FileName))
                MakeCellImages()
                Shuffle()
            End If
        Catch ex As Exception
            MessageBox.Show("ファイルの読み込みに失敗しました" & vbCrLf & ex.Message, Me.Text, MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try

    End Sub

    Sub MakeMainImage(ByVal b As Image)

        Dim w As Integer = (b.Width \ XCount) * XCount + 1
        Dim h As Integer = (b.Height \ YCount) * YCount + 1

        MainImage = b
        SourceImage = New Bitmap(w, h)
        Dim g As Graphics = Graphics.FromImage(SourceImage)
        g.DrawImage(b, New Point(0, 0))

    End Sub

    Private Sub ExitToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ExitToolStripMenuItem.Click

        Me.Dispose()

    End Sub
End Class
